/***************************************************************************
                          cmdlineoptions.cpp  -  description
                             -------------------
    begin                : Sun Nov 25 2001
    copyright            : (C) 2001 by Andr Simon
    email                : andre.simon1@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cmdlineoptions.h"
#include "stringtools.h"
#include "configurationreader.h"
#include "datadir.h"
#include <stdlib.h>
#include <string>
#include <iostream>
#include <fstream>

#ifndef _WIN32
#include <getopt.h>
#else
#include "win32cmdline.h"
#endif

#include "ASFormatter.h"

using namespace std;
using namespace astyle;

/* Siehe man getopt (3)
   Konstruktor legt Optionen und Argumente fest
*/
CmdLineOptions::CmdLineOptions(int argc, char *argv[]):
    formatStyle(DISABLED),
    cssOutFilename("highlight.css"),
    cssInFilename(""),
    opt_outfile (false),
    opt_infile (false),
    opt_language (false),
    opt_include_css (false),
    opt_help  (false),
    opt_version  (false),
    opt_verbose (false),
    opt_linenumbers (false),
    opt_css_style (false),
    opt_css_outfilename (false),
    opt_css_infilename (false),
    opt_delete_tabs (false),
    opt_xhtml_output (false),
    opt_batch_mode (false),
    opt_batch_recursive(false),
    opt_rtf_output (false),
    opt_tex_output (false),
    opt_latex_output (false),
    opt_fragment (false) ,
    opt_attach_line_anchors (false),
    opt_show_themes (false),
    opt_show_langdefs (false),
    opt_outdir(false),
    opt_asformat_output(false),
    opt_data_dir(false),
    opt_printindex(false),
    helpmsg("en")
{

  loadPresets();

#ifndef _WIN32

  int c, option_index = 0;

  static struct option long_options[] =
      {
        {"output", 1, 0, 'o'},
        {"input", 1, 0, 'i'},
        {"syntax", 1, 0, 'S'},
        {"verbose", 0, 0, 'v'},
        {"includecss", 0, 0, 'I'},
        {"help", 0, 0, 'h'},
        {"help-int", 1, 0, 'H'},
        {"linenumbers",0,0,'l'},
        {"style", 1,0,'s'},
        {"css-outfile", 1, 0,'c'},
        {"css-infile", 1, 0,'e'},
        {"deletetabs",1,0,'t'},
        {"hilfe", 0,0,'d'},        
        {"xhtml", 0,0,'X'},
        {"rtf", 0,0,'R'},
        {"tex",0, 0,'T'},
        {"latex",0, 0,'L'},
        {"batch",1,0,'b'},
        {"batch-recursive",1,0,'B'},        
        {"fragment",0,0,'f'},
        {"anchors", 0,0,'a' },
        {"listthemes", 0,0,'w' },
        {"listlangs", 0,0,'p' },
        {"outdir",1,0,'O'},
        {"version",0,0,0},
        {"format-style",1,0,'F'},
        {"data-dir",1,0,'D'},
        {"printindex",0,0,'C'},
        {0, 0, 0, 0}
      };
#else

    CommandLineParser p;
	p.addSwitchOption('v', "verbose");
	
	p.addSwitchOption('I', "includecss");
	p.addSwitchOption('h', "help");
    p.addSwitchOption('d', "hilfe");
    p.addSwitchOption('l', "linenumbers");
	p.addSwitchOption('X', "xhtml");
	p.addSwitchOption('R', "rtf");      
	p.addSwitchOption('T', "tex");      
	p.addSwitchOption('L', "latex");      
	p.addSwitchOption('f', "fragment");        	
	p.addSwitchOption('a', "anchors");      
	p.addSwitchOption('w', "listthemes");
	p.addSwitchOption('p', "listlangs");      
	p.addSwitchOption('Q', "version");      
    p.addStringOption('i', "input");	
	p.addStringOption('o', "output");	
	p.addStringOption('S', "syntax");	
	p.addStringOption('s', "style");	
    p.addStringOption('c', "css-outfile");	
    p.addStringOption('e', "css-infile");	
    p.addIntOption('t', "deletetabs");	
    p.addStringOption('b', "batch");	
    p.addStringOption('B', "batch-recursive");	
    p.addStringOption('O', "outdir");	
    p.addStringOption('F', "format-style");	
    p.addStringOption('D', "data-dir");
    p.addStringOption('C', "printindex");	
    p.addStringOption('H', "help-int");	
    try
	{
		p.parse(argc, argv);	
	}
	catch (ArgumentParseException& ex)
	{
		cerr << ex.what() << endl;
		//return -1;
	}
#endif      

#ifndef _WIN32  
  while (1)
    {
      c = getopt_long (argc, argv,
                       "o:i:S:b:B:O:adfhlvwpILRTXs:c:e:t:F:D:CH:",
                       long_options, &option_index);
      if (c == -1)
        break;

      switch (c)
        {
        case 0:   //nur long options
          if (long_options[option_index].name=="version")
            {
              opt_version = true;
            }
          break;
        case 'o':
          outFilename=string(optarg);
          opt_outfile = true;
          break;
        case 'i':
          inFilename=string(optarg);
          if (!opt_language)
            {
              language=getFileSuffix(inFilename);
            }
          opt_infile = true;
          
          break;
        case 'c':
          cssOutFilename=string(optarg);
          opt_css_outfilename = true;
          break;
        case 'e':
          cssInFilename=string(optarg);
          opt_css_infilename = true;
          break;
        case 'v':
          opt_verbose = true;
          break;
        case 'I':
          opt_include_css = true;
          break;
        case 'd':
          helpmsg="de";
          opt_help = true;
          break;
        case 'H':
          helpmsg=string(optarg);
        case 'h':        
          opt_help = true;
          break;
        case 'l':
          opt_linenumbers = true;
          break;
        case '?':
          opt_help = true;
          break;
        case 's':
          styleName=string(optarg);
          opt_css_style = true;
          break;
        case 'S':
          language=string(optarg);
          opt_language = true;
          break;
        case 't':
          opt_delete_tabs = true;
          numberSpaces = StringTools::str2int (optarg);
          break;
        case 'X':
          opt_xhtml_output = true;
          break;
        case 'R':
          opt_rtf_output = true;
          break;
        case 'T':
          opt_tex_output = true;
          break;
        case 'L':
          opt_latex_output = true;
          break;
        case 'B':
          opt_batch_recursive = true;  
        case 'b':
          batchwildcard = "*." + string(optarg);
          language= string( optarg);
          opt_batch_mode = true;
          break;
        case 'f':
          opt_fragment = true;
          break;
        case 'a':
          opt_attach_line_anchors = true;
          break;
        case 'w':
          opt_show_themes = true;
          break;
        case 'p':
          opt_show_langdefs = true;
          break;

        case 'O':
          outDirectory = string(optarg);
          if (outDirectory[outDirectory.length()-1] !='/')
            {
              outDirectory += "/";
            }
          opt_outdir = true;
          break;
        case 'F':
          setFormatStyle(optarg);
          break;
        case 'D':
          dataDir=string(optarg);
          if (dataDir[dataDir.length()-1] !='/')
            {
              dataDir += "/";
            }
          opt_data_dir=true;
          break;
         case 'C':
          opt_printindex=true;
          break;
        default:
          cerr <<"?? unknown option " <<c<< endl;
        }
    }
#else

 opt_version = p["Q"];

  if (p.hasOption("o")){
          outFilename=p["output"];
          opt_outfile = true;
   }

  if (p.hasOption("i")){
          inFilename=p["input"];
          if (!opt_language)
            {
              language=getFileSuffix(inFilename);
            }
          opt_infile = true;
   }
   if (p.hasOption("c")){
          cssOutFilename=p["css-outfile"];
          opt_css_outfilename = true;
   }
   if (p.hasOption("e")){
          cssInFilename=p["css-infile"];
          opt_css_infilename = true;
   }
   opt_verbose = p["v"];
   opt_include_css = p["I"];
   
   if ((bool)p["d"]){
           helpmsg="de";
   }
   opt_help = (bool)p["h"] || (bool)p["d"];

   opt_linenumbers = p["l"];

   if (p.hasOption("H")){
          helpmsg=p["help-int"];
          opt_help = true;
   }
   if (p.hasOption("s")){
          styleName=p["style"];
          opt_css_style = true;
   }
   if (p.hasOption("S")){
          language=p["syntax"];
          opt_language = true;
    }
   if (p.hasOption("t")){
          opt_delete_tabs = true;
          numberSpaces = p["deletetabs"];
   }
   opt_xhtml_output = p["X"];
   opt_rtf_output = p["R"];
   opt_tex_output = p["T"];
   opt_latex_output = p["L"];

   if (p.hasOption("B")){
      opt_batch_recursive = true;
      batchwildcard = "*." + (string)p["batch-recursive"];
      language= p["batch-recursive"];
      opt_batch_mode = true;
   }
  if (p.hasOption("b")){
      batchwildcard = "*." + (string)p["batch"];
      language= p["batch"];
      opt_batch_mode = true;
   }

   opt_fragment = p["f"];
   opt_attach_line_anchors = p["a"];
   opt_show_themes = p["w"];
   opt_show_langdefs = p["p"];
   	
   opt_printindex=p["C"];

   if (p.hasOption("O")){
        outDirectory = p["outdir"];
        if (outDirectory[outDirectory.length()-1] !='\\')
            {
              outDirectory += "\\";
            }
          opt_outdir = true;
       }
    
    if (p.hasOption("F")){
       string f=p["format-style"];
       setFormatStyle(f.c_str());
    }
    
    if (p.hasOption("D")){
       dataDir=string(p["data-dir"]);
       if (dataDir[dataDir.length()-1] !='\\')
          {
           dataDir += "\\";
         }
       opt_data_dir=true;
    }	
#endif    

#ifndef _WIN32
  if (optind < argc)   //still args left
#else
  if (argc > 1)
#endif
    {
      //Inputfile noch unbekannt ?
      if  (!opt_infile)
        {
#ifndef _WIN32        
          inFilename=argv[optind++];
#else
          inFilename=argv[1];
#endif          
          if (!opt_language)
            {
              language=getFileSuffix(inFilename);
            }
          opt_infile=true;
        }
      else
        {
          errmsg="Inputfile (" + inFilename + ") already defined.\n\n";
        }
    }  
}

CmdLineOptions::~CmdLineOptions(){
}

void CmdLineOptions::setFormatStyle(const char * style)
{
  string arg (style);
  opt_asformat_output =  true;

if ( arg =="ansi")
    {
        formatStyle =ANSI;
    }
  else if ( arg =="gnu")
    {
        formatStyle = GNU;
    }
  else if ( arg =="java")
    {
        formatStyle = JAVA;
    }
  else if ( arg =="kr")
    {
        formatStyle = KR;
    }
  else if ( arg =="linux")
    {
        formatStyle = LINUX;
    }
  else
    {
      opt_asformat_output =  false;
      cerr << " highlight: Unknown reformatting argument (-F): "
      << arg
      << "."<<endl;
    }  
}

/** gibt Dateiendung zurck, wenn kein "." gefunden wird, wird Leerstring ""
    zurckgegeben */
string CmdLineOptions::getFileSuffix(const string &fileName) const
  {
    unsigned int ptPos=fileName.rfind(".");
    return (ptPos == string::npos) ? "" : fileName.substr(ptPos+1, fileName.length());
  }

/** gibt bei mehreren gebraeuchlichen Dateiendungen den passenden Namen der
Sprachdefinitionsdatei zurueck */
string CmdLineOptions::checkFileType( string suffix)
{
  if (extensions[ suffix ] != "")
    {
      return extensions[ suffix ];
    }
  return suffix;
}

const string &CmdLineOptions::getOutFilename() 
  {
   int delim;
#ifdef WIN32
    delim = inFilename.find_last_of('\\');
#else
    delim = inFilename.find_last_of('/');
#endif
    if (opt_infile && opt_outdir && !opt_outfile){
      outFilename = outDirectory +
                   inFilename.substr((delim>-1)?delim:0) 
                   + getOutFileSuffix();
    }
    return outFilename;
  }

const string &CmdLineOptions::getInFilename()  const
  {
    return inFilename;
  }

const string &CmdLineOptions::getOutDirectory()const
  {
    return outDirectory;
  }

string CmdLineOptions::getLanguage() 
  {
    language=checkFileType(language);
    return language + ".lang";
  }
const string &CmdLineOptions::getCssOutFilename() const
  {
    return cssOutFilename;
  }
const string &CmdLineOptions::getCssInFilename() const
  {
    return cssInFilename;
  }
const string &CmdLineOptions::getErrMsg() const
  {
    return errmsg;
  }
const string& CmdLineOptions::getBatchWildcard()const
  {
    return batchwildcard;
  }
int CmdLineOptions::getNumberSpaces() const
  {
    return ((numberSpaces>0) && (numberSpaces<11)) ? numberSpaces : 0;
  }
/** gibt true zurck, falls Versionsinfo ausgegeben werden soll */
bool CmdLineOptions::printVersion()const
  {
    return opt_version;
  }
/** gibt true zurck, falls ein Hilfetext ausgegeben werden soll */
bool CmdLineOptions::printHelp()const
  {
    return opt_help;
  }
/** gibt true zurck, falls Debuginfo ausgegeben werden soll */
bool CmdLineOptions::printDebugInfo()const
  {
    return opt_verbose;
  }

/** gibt true zurck, falls CSS in Ausgabedatei eingebettet werden soll */
bool CmdLineOptions::includeCssDefinitions()const
  {
    return opt_include_css;
  }
/** gibt true zurck, falls Zeilennummern ausgegeben werden sollen */
bool CmdLineOptions::printLineNumbers()const
  {
    return opt_linenumbers;
  }
/** gibt true zurck, falls XHTML-Datei ausgegeben werden soll */
bool CmdLineOptions::xhtmlFormat()const
  {
    return opt_xhtml_output;
  }
/** gibt true zurck, falls HTML-Datei ausgegeben werden soll */
bool CmdLineOptions::htmlFormat()const
  {
    return !(opt_rtf_output||opt_xhtml_output||opt_latex_output
             || opt_tex_output);
  }
/** gibt true zurck, falls RTF-Datei ausgegeben werden soll */
bool CmdLineOptions::rtfFormat()const
  {
    return opt_rtf_output;
  }
/** gibt true zurck, falls TeX-Datei ausgegeben werden soll */
bool CmdLineOptions::texFormat()const
  {
    return  opt_tex_output;
  }
/** gibt true zurck, falls LaTeX-Datei ausgegeben werden soll */
bool CmdLineOptions::latexFormat()const
  {
    return opt_latex_output;
  }
/** gibt true zurck, falls AbiWord-Datei ausgegeben werden soll */
/*bool CmdLineOptions::abiFormat()const{
  return opt_abi_output;
} */
/** gibt gewuenschten Stil aus (Default: kwrite) */
string CmdLineOptions::getStyleName()const
  {
    return ( ( opt_css_style) ? styleName+".style" : "kwrite.style" );
  }
/** gibt true zurck, falls Tabs durch Leerzeichen ersetzt werden sollen */
bool CmdLineOptions::deleteTabs()const
  {
    return opt_delete_tabs;
  }
/** gibt true zurck, falls Batchmodus aktiviert wurde */
bool CmdLineOptions::enableBatchMode()const
  {
    return opt_batch_mode;
  }

bool CmdLineOptions::fragmentOutput()const
  {
    return opt_fragment;
  }

/** gibt Dateieindung der Ausgabedatei(en) zurueck */
string CmdLineOptions::getOutFileSuffix()const
  {

    if (opt_xhtml_output)
      return ".xhtml";
    if (opt_rtf_output)
      return ".rtf";
    if  (opt_tex_output || opt_latex_output)
      return ".tex";

    return ".html";
  }
/** gibt true zurck, falls Anker an Zeilennummerierung gesetzt werden sollen */
bool CmdLineOptions::attachLineAnchors()const
  {
    return opt_attach_line_anchors;
  }

bool CmdLineOptions::showThemes()const
  {
    return opt_show_themes;
  }

bool CmdLineOptions::showLangdefs()const
  {
    return opt_show_langdefs;
  }

bool CmdLineOptions::outDirGiven()const
  {
    return opt_outdir;
  }

void CmdLineOptions::loadPresets()
{
 string paramVal;
#ifndef _WIN32
  string path=getenv("HOME");  
  path += "/.highlight/highlight.conf";
#else
  string path = "highlight.conf";
#endif  
  ConfigurationReader presets(path, false);
  if (presets.found())
    {
      paramVal = presets.getParameter("style");
      if (paramVal!="")
        {
          styleName=paramVal;
          opt_css_style = true;         
        }
      paramVal = presets.getParameter("numberspaces");
      if (paramVal!="")
        {
          opt_delete_tabs = true;
          numberSpaces = StringTools::str2int (paramVal );         
        }
      paramVal = presets.getParameter("linenumbers");      
      if (paramVal == "true")
        {
          opt_linenumbers = true;
        }
      paramVal = presets.getParameter("includecss");   
      if (paramVal == "true")
        {
          opt_include_css = true;
        }
      paramVal = presets.getParameter("cssoutfilename");
      if (paramVal != "")
        {
          cssOutFilename=paramVal;
          opt_css_outfilename = true;
        }
      paramVal = presets.getParameter("cssinfilename");
      if (paramVal != "")
        {
          cssInFilename=paramVal;
          opt_css_infilename = true;
        }
      paramVal = presets.getParameter("formatstyle");
      if (paramVal != "")
        {
          setFormatStyle(paramVal.c_str());
          opt_delete_tabs=false;
        }
      paramVal = presets.getParameter("datadir");
      if (paramVal != "")
        {
          dataDir=paramVal;
          opt_data_dir=true;          
        }
    }

}


bool CmdLineOptions::formattingEnabled() 
  {
    if (opt_asformat_output)
    {
      if (   (language=="c") ||(language=="cs")
           ||(language=="java"))
        {
          opt_delete_tabs = false;  // Tabs werden durch ASFormatter ersetzt
        }
      else
        {
          cerr << "highlight: Reformatting only enabled with C, C++, C# or Java input files."
               << endl;
          opt_asformat_output=false;
        }
    }
    return opt_asformat_output;
  }

bool CmdLineOptions::newDataDir()const {
  return opt_data_dir;  
}

/** ldt Liste der bekannten Dateiendungen */
#ifndef _WIN32
bool CmdLineOptions::loadFileExtensions(){
  ConfigurationReader extensionsConfig(DataDir::getDir() + "extensions.conf", true);
#else
bool CmdLineOptions::loadFileExtensions(const string & path){
  ConfigurationReader extensionsConfig(path + "extensions.conf", true);  
#endif
  
  if (extensionsConfig.found())
    {
      stringstream *values, *names=extensionsConfig.getParameterNames();
      string paramName, paramVal;
      while ( *names >> paramName)
        {
          values =  new stringstream(extensionsConfig.getParameter(paramName).c_str()) ;
          while (*values >> paramVal)
            {
              extensions[ paramVal ] = paramName;
            }
          delete values;
        }
      return true;
    }
  return false;

}
/** gibt Datenverzeichnis zurck */
string & CmdLineOptions::getDataDir() {
 return dataDir;
}
/** true, falls Verzeichnisse rekursiv durchsucht werden sollen */
bool CmdLineOptions::batchRecursive() const{
 return opt_batch_recursive;
}
bool CmdLineOptions::printIndexFile() const{
 return opt_printindex && (htmlFormat() || xhtmlFormat());
}
/** No descriptions */
enum FormatStyle CmdLineOptions::getFormatStyle(){
  return formatStyle;
}
const string & CmdLineOptions::getHelpMsg()const{
  return helpmsg;
}
