/***************************************************************************
                          cmdlineoptions.h  -  description
                             -------------------
    begin                : Sun Nov 25 2001
    copyright            : (C) 2001 by Andr Simon
    email                : andre.simon1@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CMDLINEOPTIONS_H
#define CMDLINEOPTIONS_H

#include <string>
#include <map>
#include "stringtools.h"
#include "ASFormatter.h"

using namespace std;
   

typedef map<string, string> ExtensionsMap;



class CmdLineOptions
  {
  public:

    //Konstruktor erhaelt Kommandozeilenparameter
    CmdLineOptions(int argc, char *argv[]);
    ~CmdLineOptions();
    
    /** gibt Namen der Ausgabedatei zurueck*/
    const string &getOutFilename();
    /** gibt Namen der Eingabedatei zurueck*/
    const string&getInFilename() const;
    /** gibt Namen des Ausgabeverzeichnisses zurueck*/
    const string&getOutDirectory() const;
    /** gibt Namen der Sprchdefinitionsdatei zurueck*/
    string getLanguage();
    /** gibt Namen der CSS-Ausgabedatei zurueck*/
    const string&getCssOutFilename() const;
    /** gibt Namen der CSS-Eingabedatei zurueck*/
    const string&getCssInFilename() const;
    /** gibt Fehlermeldung zurueck*/
    const string&getErrMsg() const;
    /** gibt an, durch wieviele Leerzeichen ein Tab ersetzt werden soll*/
    const string&getBatchWildcard() const;
    /** gibt Anzahl der Leerzeichen zurueck, die Tabs ersetzen sollen*/
    int getNumberSpaces() const;
    /** gibt true zurck, falls Versionsinfo ausgegeben werden soll */
    bool printVersion() const;
    /** gibt true zurck, falls ein Hilfetext ausgegeben werden soll */
    bool printHelp() const;
    /** gibt true zurck, falls Debuginfo ausgegeben werden soll */
    bool printDebugInfo()const;
    /** gibt true zurck, falls CSS in Ausgabedatei eingebettet werden soll */
    bool includeCssDefinitions() const;
    /** gibt true zurck, falls Zeilennummern ausgegeben werden sollen */
    bool printLineNumbers() const;
    /** gibt gewuenschten Themenamen aus */
    string getStyleName()const ;

    /** gibt true zurck, falls bestimmtes Format ausgegeben werden soll */
    bool xhtmlFormat() const;
    bool rtfFormat() const;
    bool texFormat() const;
    bool latexFormat() const;
    bool htmlFormat() const;

    /** gibt true zurck, falls deutsche Hilfe ausgegeben werden soll */
    int helpLanguage() const;
    /** gibt true zurck, falls Tabs durch Leerzeichen ersetzt werden sollen */
    bool deleteTabs() const;
    /** gibt true zurck, falls Batchmodus aktiviert wurde */
    bool enableBatchMode() const;
    /** gibt true zurck, falls Header und Footer-Teil der Ausgabedatei
        weggelassen werden sollen */
    bool fragmentOutput() const;

    /** gibt Dateieindung der Ausgabedatei(en) zurueck */
    string getOutFileSuffix() const;
    /** gibt true zurck, falls Anker an Zeilennummerierung gesetzt werden sollen */
    bool attachLineAnchors() const;
    /** gibt true zurck, falls installierte Style-Themes angezeigt werden sollen */
    bool showThemes() const;
    /** gibt true zurck, falls installierte lang-Dateien angezeigt werden sollen */
    bool showLangdefs() const;

    /** gibt true zurck, falls Ausgabeverzeichnis gesetzt wurde */
    bool outDirGiven() const;

    bool formattingEnabled();

    bool newDataDir()const;
    bool printIndexFile() const;
    
  /** ldt Liste der bekannten Dateiendungen */
#ifdef _WIN32  
  bool loadFileExtensions(const string &);
#else
   bool loadFileExtensions();
#endif   
  /** gibt Datenverzeichnis zurck */
  string & getDataDir();
  /** true, falls Verzeichnisse rekursiv durchsucht werden sollen */
  bool batchRecursive() const;
  /** No descriptions */
  enum FormatStyle getFormatStyle();

  const string & getHelpMsg() const;

  private:

    int numberSpaces;   //anzahl Spaces, die einen Tabulator ersetzen sollen
    enum FormatStyle formatStyle;
    string errmsg;  //Fehlermeldung
    

    string outFilename, //Name der Ausgabedatei
    inFilename,     //Name der Eingabedatei
    outDirectory,   //Zielverzeichnis fuer Ausgabedateien
    language,       //Programmiersprache, die geparst wird
    styleName,      //Stil der formatierung (zB kwrite oder emacs)
    cssOutFilename, //Name der CSS-Ausgabedatei
    cssInFilename,  //Name der CSS-Eingabedatei
    batchwildcard,  //Dateiendung der Dateien, die im Batch-Mode geparst
                    //werden sollen    
    dataDir;        //Datenverzeichnis

    // Zuordnung von Dateiendungen zu einer Syntaxdefinition
    ExtensionsMap extensions;

    //Flags zur Bestimmung der gesetzten Parameter:
    bool opt_outfile;
    bool opt_infile;
    bool opt_language;
    bool opt_include_css;
    bool opt_help;
    bool opt_version ;
    bool opt_verbose;
    bool opt_linenumbers;
    bool opt_css_style;
    bool opt_css_outfilename;
    bool opt_css_infilename;
    bool opt_delete_tabs;
    bool opt_xhtml_output;
    bool opt_batch_mode,
         opt_batch_recursive;
    bool opt_rtf_output;
    bool opt_tex_output;
    bool opt_latex_output;
    bool opt_fragment;
    bool opt_attach_line_anchors;
    bool opt_show_themes;
    bool opt_show_langdefs;
    bool opt_outdir;
    bool opt_asformat_output;
    bool opt_config_dir,
         opt_data_dir,
         opt_printindex;

    string helpmsg;

    /** ldt Voreinstellungen aus Configdatei .highlight.conf im HOME Verzeichnis
        Kommandozeilenparameter berschreiben diese, falls angegeben */
    void loadPresets();
        

    /** gibt Dateiendung zurck, wenn kein "." gefunden wird, wird
        "" zurckgegeben */    
    string getFileSuffix( const string & fileName) const;

    /** gibt bei mehreren gebraeuchlichen Dateiendungen den passenden Namen der
        Sprachdefinitionsdatei zurueck */
    string checkFileType( string suffix);
    
    void setFormatStyle(const char * );

  };

#endif

