/***************************************************************************
                          configurationreader.cpp  -  description
                             -------------------
    begin                : Son Nov 10 2002
    copyright            : (C) 2002 by Andr Simon
    email                : andre.simon1@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/*
 Liest Werte aus Configdatei mit folgender Formatierung ein:
 /ParamName=ParamWert
 ParamWert kann sich dabei ueber mehrere Zeilen erstrecken,  bis ein neuer
 ParamName erscheint.
 
 # ist Kommentar, falls es das erste Zeichen der Zeile ist
*/


#include "configurationreader.h"
#include "stringtools.h"
#include <string>
#include <iostream>
#include <fstream>

using namespace std;

ConfigurationReader::ConfigurationReader(const string & configuration_path, bool complain)
{
  ifstream in (configuration_path.c_str());
  if (in)
    {
      string  line;
      line.reserve(1000);
      unsigned int trennerPos, zeilenBeginn;
      string paramName, paramValue;
      while (getline(in, line))
        {
          zeilenBeginn=line.find_first_not_of("\t ");
          if ((line.size()>2) &&(zeilenBeginn!=string::npos) && (line.at(zeilenBeginn)!='#'))  //Kommentar??
            {
              if (line.at(zeilenBeginn)=='/')
                {             // neuer Parametername?
                  trennerPos=line.find("=",zeilenBeginn)-1;  // Name vom Wert trennen
                  if (trennerPos!=string::npos)
                    {
                      paramName=line.substr(zeilenBeginn+1, trennerPos);
                      parameterNames << paramName << " ";
                      paramValue=line.substr(trennerPos+2, line.length());
                    }
                }
              else
                {                                      // Zeile gehoert zum letzten Parameterwert
                  paramValue=line;
                }
              // Wert an wert[name] haengen
              if (parameterMap[StringTools::lowerCase(paramName)]=="")
                {
                  parameterMap[StringTools::lowerCase(paramName)]
                  = paramValue;
                }
              else
                {
                  parameterMap[StringTools::lowerCase(paramName)]
                  += (" "+paramValue);
                }                        
            }  //if ((zeilenBeginn!=string::npos) && (line.at(zeilenBeginn)!='#'))
        } //while
      fileFound = true;
      in.close();
    } //if (in)
  else
    {
      if (complain)
        {
          cerr << "highlight: file "
          << configuration_path
          << " not found.\n"
          << endl;
        }
      fileFound = false;
    }
}
ConfigurationReader::~ConfigurationReader()
{}

bool ConfigurationReader::found()
{
  return fileFound;
}

/*liefert Wert des Paramters zurueck
  ParameterName ohne einleitendes "/" angeben! */
string &ConfigurationReader::getParameter(const string & paramName)
{
  return parameterMap[paramName] ;
}

stringstream  *ConfigurationReader::getParameterNames()
{
  return &parameterNames;
}
