/***************************************************************************
                          htmlcode.cpp  -  description
                             -------------------
    begin                : Wed Nov 28 2001
    copyright            : (C) 2001 by Andr Simon
    email                : andre.simon1@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "htmlcode.h"
#include "version.h"
#include <iostream>
#include <fstream>
#include <string>
#include <sstream>

using namespace std;
using namespace highlight;

// CSS-Kommentar
const string HtmlCode :: cssComment =
  "\t/* Used abbreviations: */\n"
  "\t/* key: Keywords */\n"
  "\t/* typ: Types and type modifiers */\n"
  "\t/* str: Strings */\n"
  "\t/* num: Numbers */\n"
  "\t/* com: Comments */\n"
  "\t/* esc: Escape characters */\n"
  "\t/* dir: Directive lines */\n"
  "\t/* dstr: Strings within directive lines */\n"
  "\t/* line: line numbers */\n\n";


HtmlCode::HtmlCode(void)
{}

string  HtmlCode::formatStyleAttributes(const string & elemName, StyleColour * col,ElementStyle * elem)
{
  stringstream s;
  s   << "\t."<<elemName<<"\t{ color:#"
  << (col->getHTMLRedValue())
  << (col->getHTMLGreenValue())
  << (col->getHTMLBlueValue()  )
  << ( elem->isBold() ?"; font-weight:bold" :"" )
  << ( elem->isItalic() ?"; font-style:italic" :"" )
  << ( elem->isUnderline() ?"; text-decoration:underline" :"" )
  << "; }\n" ;
  return  s.str();
}

HtmlCode::HtmlCode (
  LanguageDefinition &langDef,
  const string &cssStyle,
  const string &cssFilename,
  bool includeCssDef, bool printLineNumbers,
  bool withAnchors, bool xhtmlFormat,
  int numSpaces)
    : CodeParser(langDef, printLineNumbers, cssStyle, numSpaces),
    attachAnchors(withAnchors),
    cssFile(cssFilename),
    includeCss(includeCssDef),
    xhtmlOutput(xhtmlFormat)
{
  for (int i=1;i<NUMBER_OF_STYLES; i++)
    {
      styleTagClose[i] = "</span>";
    }
  styleTagClose[STANDARD] = "";

  // Definition der CSS-Klassen, die zum Highlighting benutzt werden:
  styleTagOpen[STANDARD] = "";
  styleTagOpen[KEYWORD] = "<span class=\"key\">";
  styleTagOpen[TYPE] = "<span class=\"typ\">";
  styleTagOpen[STRING] = "<span class=\"str\">";
  styleTagOpen[NUMBER] = "<span class=\"num\">";
  styleTagOpen[SINGLE_LINE_COMMENT] = "<span class=\"com\">";
  styleTagOpen[MULTI_LINE_COMMENT_BEGIN] = "<span class=\"com\">";
  styleTagOpen[ESC_CHAR] = "<span class=\"esc\">";
  styleTagOpen[DIRECTIVE_LINE] = "<span class=\"dir\">";
  styleTagOpen[DIRECTIVE_STRING] = "<span class=\"dstr\">";
  styleTagOpen[LINENUMBER] = "<span class=\"line\">";
  newLineTag = "\n";
  spacer = " ";
  /*
      char* tok;
      const char* delimiters =",";

      tok = strtok((char*)cssStyle.c_str(), delimiters);
      while(tok) {
       // Automatic type conversion:
       styleList.push_back(tok);
       cerr << "tok="<<tok;
       tok = strtok(0, delimiters);
      }
      cerr << "tok fertig"<<endl;
  */
}

void HtmlCode::printStyleDefinition(ostream * out)
{
  if (!docStyle->found())
    {
      *out << "/* ERROR: Style not found. */\n";
      return;
    }
  *out << "\tpre\t{ color:#"
  << (docStyle->getDefaultColour()->getHTMLRedValue())
  << (docStyle->getDefaultColour()->getHTMLGreenValue())
  << (docStyle->getDefaultColour()->getHTMLBlueValue()  )
  << "; background-color:#"
  << (docStyle->getBgColour()->getHTMLRedValue())
  << (docStyle->getBgColour()->getHTMLGreenValue())
  << (docStyle->getBgColour()->getHTMLBlueValue()  )
  <<"; font-size:"
  << docStyle->getFontSize()
  <<"pt; font-family:Courier;}\n"
  << formatStyleAttributes("key", docStyle->getKeywordColour(), docStyle->getKeywordStyle())
  << formatStyleAttributes("num",docStyle->getNumberColour(), docStyle->getNumberStyle())
  << formatStyleAttributes("typ",docStyle->getTypeColour(), docStyle->getTypeStyle())
  << formatStyleAttributes("esc",docStyle->getEscapeCharColour(), docStyle->getEscapeCharStyle())
  << formatStyleAttributes("str",docStyle->getStringColour(), docStyle->getStringStyle())
  << formatStyleAttributes("dstr", docStyle->getDirectiveStringColour(), docStyle->getDirectiveStringStyle())
  << formatStyleAttributes("com", docStyle->getCommentColour(), docStyle->getCommentStyle())
  << formatStyleAttributes("dir",docStyle->getDirectiveColour(), docStyle->getDirectiveStyle())
  << formatStyleAttributes("line", docStyle->getLineColour(), docStyle->getLineStyle());
}

//Gibt den Kopf der (X)HTML-Seite aus. Fuegt ggf. CSS-Definitionen ein.
void HtmlCode::printHeader(const string &title)
{
  if (xhtmlOutput)   //XHTML Ausgabe:
    {
      *out << "<?xml version=\"1.0\" encoding=\"iso-8859-1\"?>\n"
              "<!DOCTYPE  html PUBLIC \"-//W3C//DTD XHTML 1.1//EN\""
              "  \"http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd\">\n"
              "<html xmlns=\"http://www.w3.org/1999/xhtml\">" << endl;
    }
  else     //HTML Ausgabe:
    {
      *out << "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\">\n"
      "<html>" << endl;
    }

  *out <<"<head>\n\t<title>" << ((title=="") ? "Source file" : title)
  << "</title>" << endl;

  if (langInfo.getSyntaxHighlight())
    {
      if (includeCss)    //CSS-Definition in HTML-<head> einfuegen
        {
          *out <<	"\t<style type=\"text/css\">\n";
          if (!xhtmlOutput){
              *out<< "\t<!--\n";
          }
          //XHTML-Deklaration von PCDATA im Scriptbereich:
          *out<< ((xhtmlOutput) ? "\t<![CDATA[\n" : "")
          << cssComment;
          printStyleDefinition(out);
          *out << ((xhtmlOutput) ? "\t]]>\n" : "\t//-->\n");
          *out <<"\t</style>" << endl;
        }
      else              //Referenz auf CSS-Datei einfuegen
        {
          *out << "\t<link rel=\"stylesheet\" type=\"text/css\" href=\""
          << cssFile
          << "\""
          << ((xhtmlOutput) ? "/" : "")
          << ">"
          << endl;
          /*            for (unsigned int i=1; i< styleList.size();i++){
                        *out << "\t<link rel=\"alternate stylesheet\" type=\"text/css\" "
                             <<" title=\""<<styleList[i]<<"\" href=\""
                             << styleList[i]<<".css"
                             << "\""
                             << ((xhtmlOutput) ? "/" : "")
                             << ">"
                             << endl;
                       }*/
        }
    }

  *out << "</head>\n"
  "<body>\n"
  "<pre>"
  << endl;
}

// Gibt Ende der HTML-Datei zurueck
void HtmlCode::printFooter()
{
  *out << "\n</pre>\n</body>\n</html>\n<!--"
  << ((xhtmlOutput) ? "X" : "")
  << "HTML generated by highlight "
  << HIGHLIGHT_VERSION
  << ", "
  << HIGHLIGHT_URL
  << "-->"
  << endl;
}


// Gibt den Hauptteil der HTML-Datei zurueck
void HtmlCode::printBody()
{
  processStandardState();
}


/** schreibt Style Sheet- Definition in cssFileName */
bool HtmlCode::printCSSFile(const string &cssInFileName, const string &cssOutFileName)
{
  if (!includeCss && langInfo.getSyntaxHighlight())
    {
      ofstream cssFile(cssOutFileName.c_str());
      if (cssFile)
        {
          cssFile << "/* CSS definition file generated by highlight "
          << HIGHLIGHT_VERSION
          << ", "
          << HIGHLIGHT_URL
          << " */\n\n";

          if (cssInFileName!="")
            {
              ifstream cssUserDef(cssInFileName.c_str());
              if (cssUserDef)
                {
                  cssFile << "/* Content of "<<cssInFileName<<": */\n";
                  string line;
                  while (getline(cssUserDef, line))
                    {
                      cssFile << line;
                      cssFile << endl;
                    }
                  cssUserDef.close();
                }
              else
                {
                  cssFile <<"/* ERROR: Could not include "<<cssInFileName<<".*/"<<endl;
                }
            }

          cssFile << "\n\n/* Highlighting theme definition: */\n"
          << endl
          << cssComment;
          printStyleDefinition(&cssFile);
          cssFile << endl;
          cssFile.close();
        }
      else
        {
          return false;
        }
    }
    return true;
}

/** Gibt s mit HTML-Code der Sonderzeichen zurueck */
string HtmlCode::maskCharacter( char c)
{
  switch (c)
    {
    case '<' :
      return "&lt;";
      break;
    case '>' :
      return "&gt;";
      break;
    case '&' :
      return "&amp;";
      break;
    case '\"' :
      return "&quot;";
      break;
    case '' :
      return "&auml;";
      break;
    case '' :
      return "&ouml;";
      break;
    case '' :
      return "&uuml;";
      break;
    case '' :
      return "&Auml;";
      break;
    case '' :
      return "&Ouml;";
      break;
    case '' :
      return "&Uuml;";
      break;
    case '' :
      return "&szlig;";
      break;
    case '':
      return "&agrave;";
      break;
    case '' :
      return "&aacute;";
      break;
    case '':
      return "&egrave;";
      break;
    case '':
      return "&eacute;";
      break;
    case '':
      return "&ograve;";
      break;
    case '':
      return "&oacute;";
      break;
    case '':
      return "&ugrave;";
      break;
    case '':
      return "&uacute;";
      break;
    case '':
      return "&Agrave;";
      break;
    case '':
      return "&Aacute;";
      break;
    case '':
      return "&Egrave;";
      break;
    case '':
      return "&Eacute;";
      break;
    case '':
      return "&Ograve;";
      break;
    case '':
      return "&Oacute;";
      break;
    case '':
      return "&Ugrave;";
      break;
    case '' :
      return "&Uacute;";
      break;

    default :
      string m;
      m += c;

      return m;
    }
}

/** fuegt am Anfang der neuen Zeile eine fortlaufende Nummer ein
Parameter: State envokingState
           Zustand des Aufrufers
*/
void HtmlCode::insertLineNumber (State envokingState)
{
  if (showLineNumbers)
    {
      // Falls Zustand != Standardzustand, entsprechende Tags schliessen...
      if (envokingState != STANDARD)
        {
          *out << styleTagClose[envokingState];
        }
      if (attachAnchors)
        {
          *out << "<a "
               << ((xhtmlOutput)?"id": "name")
               <<"=\"l_" << lineNumber<<"\">";
        }
      *out <<  styleTagOpen[LINENUMBER] << StringTools::int2str(lineNumber, 4, spacer)
           << spacer << styleTagClose[LINENUMBER];
      if (attachAnchors)
        {
          *out << "</a>";
        }

      // ... und wieder oeffnen
      if (envokingState != STANDARD)
        {
          *out << styleTagOpen[envokingState];
        }
    }
}

/** schreibt Index-Datei ins Ausgabeverzeichnis */
void HtmlCode::printIndexFile(const vector<string> &fileList, const string &outPath ){
  string suffix= (xhtmlOutput)?".xhtml":".html";
  string outFilePath=outPath + "index"+ suffix;

  bool indexFileOk=assignOutStream(outFilePath);

  if (indexFileOk){

     string outFileName;
     string::size_type pos;

     printHeader("Source Index");
     *out << "<h1> Source Index</h1>\n"
          << ((xhtmlOutput)?"<hr />":"<hr>")
          <<  "<ul>"<<endl;

     for (unsigned int i=0; i < fileList.size(); i++){
         pos=
     #ifdef _WIN32
         (fileList[i]).find_last_of('\\');
     #else
         (fileList[i]).find_last_of('/');
     #endif
         outFileName =(fileList[i]).substr(pos+1);

          *out << "<li><a href=\""<< outFileName<<suffix<<"\">";
          *out << outFileName<<suffix <<"</a></li>"<<endl;
     }
     *out << "</ul>"
          << ((xhtmlOutput)?"<hr />":"<hr>")
          <<((xhtmlOutput)?"<br />":"<br>" )
          << "<small>Generated by highlight "
          << HIGHLIGHT_VERSION
          << ", "
          << "<a href=\""<<HIGHLIGHT_URL << "\" target=\"new\">"
          << HIGHLIGHT_URL << "</a></small>"
          << endl;

     printFooter();

     delete out;
  }

}
