/***************************************************************************
                          languagedefinition.cpp  -  description
                             -------------------
    begin                : Wed Nov 28 2001
    copyright            : (C) 2001 by Andr Simon
    email                : andre.simon1@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "stringtools.h"
#include "languagedefinition.h"
#include "datadir.h"
#include "codeparser.h"
#include "configurationreader.h"
#include <string>
#include <iostream>
#include <fstream>
#include <iterator>
#include <sstream>


using namespace std;

namespace highlight {
//Liest Sprachdefinitionsdatei ein, fuellt die keys-und type-Mengen
//und bildet den Symbolstring.

LanguageDefinition::LanguageDefinition():
    caseSensitive(true),
    disableHighlighting(false),
    thisIs4th(false),
    allowNestedMultiLineComments(true),
    enableTypesWithinStrings(false)
{}

void LanguageDefinition::printKeys()
{
  copy(keys.begin(), keys.end(),
       ostream_iterator<string>(cerr, " "));
}
void LanguageDefinition::printTypes()
{
  copy(types.begin(), types.end(),
       ostream_iterator<string>(cerr, " "));
}

/** gibt true zurueck, wenn s Element von keys ist */
bool LanguageDefinition::isKeyword(const string &s)
{
  if (s.length())
    {
      return ((keys.count(s)>0) || (s[0]==keywordPrefix  ));
    }
  else
    {
      return false;
    }
}

/** gibt true zurueck, wenn s Element von types ist */
bool LanguageDefinition::isType(const string &s)
{
  if (s.length())
    {
      return ((types.count(s)>0) || (s[0]==variablePrefix ) );
    }
  else
    {
      return false;
    }

}

bool LanguageDefinition::isPrefix( char c)
{
  return ( (c==variablePrefix) || (c==keywordPrefix));
}

bool LanguageDefinition::isNumberPrefix( char c)
{
  return  (c==numberPrefix);
}

/** ldt die Sprachdefinition */
bool LanguageDefinition::load(const string& lang)
{
  reset();
  definitionPath = DataDir::getLangDefDir() + lang;
  ConfigurationReader langDef(definitionPath, false);
  stringstream symbolStrStream;
  if (langDef.found())
    {
      disableHighlighting=false;
      bool valExists=false;
      string token;

      //Stringstream zum Einlesen der Token:
      istringstream valueStream;

      valueStream.str(langDef.getParameter("keywords"));
      while (valueStream >> token)
        keys.insert(token);
      valueStream.clear();

      valueStream.str(langDef.getParameter("typesmods"));
      while ( valueStream >> token)
        types.insert(token);
      valueStream.clear();

      valueStream.str(langDef.getParameter("escchar"));
      while (valueStream >> token)
        {
          symbolStrStream << " " << token <<" ";
          valExists = true;
        }
      if (valExists)
        {
          symbolStrStream << " "<<ESC_CHAR;
          valExists=false;
        }
      valueStream.clear();

      valueStream.str(langDef.getParameter("stringdelimiters"));
      while (valueStream >> token )
        {
          symbolStrStream << " " << token <<" ";
          stringCloseDelim.insert(make_pair(token, token)); // [token]=token;
          valExists=true;
        }
      if (valExists)
        {
          symbolStrStream << STRING<<" ";
          valExists=false;
        }
      valueStream.clear();

      valueStream.str(langDef.getParameter("singlelinecomment"));
      while (valueStream >> token )
        {
          symbolStrStream << " " << token << " ";
          valExists=true;
        }
      if (valExists)
        {
          symbolStrStream << SINGLE_LINE_COMMENT<<" " ;
          valExists=false;
        }
      valueStream.clear();

      valueStream.str(langDef.getParameter("multilinecomment"));
      //momentan nur 2 Token!
      if (valueStream >> token)
        {
          symbolStrStream << " "<<token <<" " << MULTI_LINE_COMMENT_BEGIN;
          valueStream >> token;
          symbolStrStream <<" "<< token<<" "<< MULTI_LINE_COMMENT_END;
        }
      valueStream.clear();

      valueStream.str(langDef.getParameter("tagdelimiters"));
      //momentan nur 2 Token!

      if (valueStream >> token)
        {
          symbolStrStream << " "<<token <<" " << TAG_BEGIN;
        }
      if (valueStream >> token)
        {
          symbolStrStream <<" "<< token<<" "<< TAG_END;
        }
      valueStream.clear();

      valueStream.str(langDef.getParameter("memberattrdelimiters"));
      if (valueStream >> token)
        {
          symbolStrStream << " "<<token <<" " << MEMBER_ATTRIBUTE_BEGIN;
        }
      if (valueStream >> token)
        {
          symbolStrStream <<" "<< token<<" "<< MEMBER_ATTRIBUTE_END;
        }
      valueStream.clear();

      valueStream.str(langDef.getParameter("iscasesensitive"));
      if ((valueStream >> token) &&(StringTools::lowerCase(token)=="true") || (StringTools::lowerCase(token)=="yes"))
        caseSensitive=true;
      valueStream.clear();

      valueStream.str(langDef.getParameter("allownestedmultilinecomments"));
      if ((valueStream >> token) &&(StringTools::lowerCase(token)=="false") || (StringTools::lowerCase(token)=="no"))
        allowNestedMultiLineComments=false;
      valueStream.clear();

      valueStream.str(langDef.getParameter("esccharsoutsidestrings"));
      if ((valueStream >> token) &&(StringTools::lowerCase(token)=="true") || (StringTools::lowerCase(token)=="yes"))
        escCharsAllowedOutsideStrings=true;
      valueStream.clear();

      valueStream.str(langDef.getParameter("thisis4th"));
      if ((valueStream >> token) &&(StringTools::lowerCase(token)=="true") || (StringTools::lowerCase(token)=="yes"))
        thisIs4th=true;
      valueStream.clear();

      valueStream.str(langDef.getParameter("highlighting"));
      if ((valueStream >> token) &&StringTools::lowerCase(token)=="no")
        disableHighlighting=true;
      valueStream.clear();

      valueStream.str(langDef.getParameter("enabletypeswithinstrings"));
      if ((valueStream >> token) &&StringTools::lowerCase(token)=="true")
        enableTypesWithinStrings=true;
      valueStream.clear();


      valueStream.str(langDef.getParameter("directive"));
      if (valueStream >> token)
        {
          symbolStrStream<<" "<<token<<" "<<DIRECTIVE_LINE;
        }
      valueStream.clear();

      valueStream.str(langDef.getParameter("variableprefix"));
      valueStream >> variablePrefix;
      valueStream.clear();

      valueStream.str(langDef.getParameter("keywordprefix"));
      valueStream >> keywordPrefix;
      valueStream.clear();

      valueStream.str(langDef.getParameter("numberprefix"));
      valueStream >> numberPrefix;
      valueStream.clear();

      valueStream.str(langDef.getParameter("simplifiedstringprefix"));
      valueStream >> simplifiedStringPrefix;
      valueStream.clear();

      valueStream.str(langDef.getParameter("allowedchars"));
      while (valueStream >> token )
        {
          allowedChars += token;
        }

      valueStream.clear();

      valueStream.str(langDef.getParameter("stringdelimiterpair"));
      while (valueStream >> token)    // String open delimiter
        {
          symbolStrStream << " "<<token <<" " << STRING;

          string token2;
          if (valueStream >> token2)   // String close delimiter
           {
              symbolStrStream <<" "<< token2<<" "<< STRING_END;
           }
          stringCloseDelim.insert(make_pair(token, token2));
         }
      valueStream.clear();

      valueStream.str(langDef.getParameter("directiveend"));
      while (valueStream >> token )
        {
          symbolStrStream << " " << token << " ";
          valExists=true;
        }
      if (valExists)
        {
          symbolStrStream << DIRECTIVE_LINE_END <<" " ;
          valExists=false;
        }

      symbolString = symbolStrStream.str();
      return true;
    }
  else
    {   //Sprachdef. datei nicht gefunden
      disableHighlighting =true;
      return false;
    }
}



/** setzt Membervariablen auf Defaultwerte */
void LanguageDefinition::reset()
{
  keys.clear();
  types.clear();
  caseSensitive= escCharsAllowedOutsideStrings =false;
  allowNestedMultiLineComments=enableTypesWithinStrings= false;
  variablePrefix = keywordPrefix= numberPrefix=simplifiedStringPrefix='\0';
}
/** gibt true zurck, falls Forth Code geparst wird */
bool LanguageDefinition::iAmForth()
{
  return thisIs4th;
}

/** gibt true zurck, falls verschachtelte MultiLineComments erlaubt sind */
bool LanguageDefinition::allowNestedMLComments(){
  return allowNestedMultiLineComments;
}

bool LanguageDefinition::highlightingDisabled(){
  return disableHighlighting;
}

bool LanguageDefinition::typesWithinStringsEnabled(){
  return enableTypesWithinStrings;
}



bool LanguageDefinition::isMatchingStringDelim(const string &open,const string &close){

//cerr << "open="<<open << " close="<< close<<endl;
//cerr << (*stringCloseDelim.find(open)).first<<"=="<<close<<"? "<<((*stringCloseDelim.find(open)).first==close)<<endl;

  return   (stringCloseDelim.count(open)? (*stringCloseDelim.find(open)).second==close :false);
 }
}
