/***************************************************************************
                          LatexCode.cpp  -  description
                             -------------------
    begin                : Mit Jul 24 2002
    copyright            : (C) 2002 by Andr Simon
    email                : andre.simon1@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "latexcode.h"
#include "stringtools.h"
#include "version.h"
#include <iostream>
#include <sstream>

using namespace highlight;

string  LatexCode::formatStyleAttributes(const string & elemName, StyleColour * col,ElementStyle * elem)
{
  stringstream s;
  s   << "\\newcommand{\\hl"
  << elemName
  << "}[1]{\\textcolor[rgb]{"
  << col->getLatexRedValue() << ","
  << col->getLatexGreenValue() << ","
  << col->getLatexBlueValue()
  << "}{";
  if (elem !=NULL){
    if (elem->isBold())
      s << "\\bf{";
    if (elem->isItalic())
      s << "\\it{";
  }
    s  <<"#1";
  if (elem !=NULL){
    if (elem->isBold())
      s << "}";
    if (elem->isItalic())
      s << "}";
    }
  s  <<"}}\n";
  return s.str();
}

LatexCode::LatexCode(LanguageDefinition &langDef, const string &cssStyle,
                     bool printLineNumbers, int numSpaces)
    : CodeParser(langDef, printLineNumbers, cssStyle, numSpaces),
      isOpeningStringDelim(true)
{
  styleTagOpen[STANDARD] = "\\hlstd{";
  styleTagOpen[KEYWORD] = "\\hlkey{";
  styleTagOpen[TYPE] = "\\hltyp{";
  styleTagOpen[STRING] = "\\hlstr{";
  styleTagOpen[NUMBER] = "\\hlnum{";
  styleTagOpen[SINGLE_LINE_COMMENT] = "\\hlcom{";
  styleTagOpen[MULTI_LINE_COMMENT_BEGIN] = "\\hlcom{";
  styleTagOpen[ESC_CHAR] = "\\hlesc{";
  styleTagOpen[DIRECTIVE_LINE] = "\\hldir{";
  styleTagOpen[DIRECTIVE_STRING] = "\\hldstr{";
  styleTagOpen[LINENUMBER] = "\\hlline{";

  for (int i=0;i<NUMBER_OF_STYLES; i++)
    {
      styleTagClose[i] = "}";
    }

  newLineTag = "\\mbox{}\\\\\n";
  spacer = "\\ ";

  maskWs=true;
  maskWsBegin = "\\hlstd{\\hskip ";
  maskWsEnd = " mm}";

  stringstream s;
  s << formatStyleAttributes("std", docStyle->getDefaultColour());
  s << formatStyleAttributes("key", docStyle->getKeywordColour(), docStyle->getKeywordStyle());
  s << formatStyleAttributes("num", docStyle->getNumberColour(), docStyle->getNumberStyle());
  s << formatStyleAttributes("typ", docStyle->getTypeColour(), docStyle->getTypeStyle());
  s << formatStyleAttributes("esc", docStyle->getEscapeCharColour(), docStyle->getEscapeCharStyle());
  s << formatStyleAttributes("str", docStyle->getStringColour(), docStyle->getStringStyle());
  s << formatStyleAttributes("dstr", docStyle->getDirectiveStringColour(), docStyle->getDirectiveStringStyle());
  s << formatStyleAttributes("com", docStyle->getCommentColour(), docStyle->getCommentStyle());
  s << formatStyleAttributes("dir", docStyle->getDirectiveColour(), docStyle->getDirectiveStyle());
  s << formatStyleAttributes("line", docStyle->getLineColour(), docStyle->getLineStyle());
  styleDefinition = s.str();
}

LatexCode::LatexCode()
{}
LatexCode::~LatexCode()
{}
/** Gibt Tex-Code der Sonderzeichen zurueck */
string LatexCode::maskCharacter( char c)
{
  string m;
  switch (c)
    {
    case '<' :
      return "$<$";
      break;
    case '>' :
      return "$>$";
      break;
    case '{':
    case '}':
    case '&':
    case '$':
    case '#':
    case '%':
      m +="\\";
      m += c;
      return m;
      break;
    case '\"':
      // wenn Code in Dokumente eingebettet wird (als Fragment),
      // werden Doppelquotes durch einfache Quotes ersetzt
      if (fragmentOutput)
        return "''";
      else  {
        return "\"";
      }
      break; 
    case '_':
      return "\\textunderscore ";
      break;
    case '^':
      return "\\textasciicircum ";
      break;
    case '\\':
      return "$\\backslash$";
      break;
    case '~':
      return "$\\sim$";
      break;
    case '|':
      return "\\textbar ";
      break;
    case '':
      return "\\\"a";
      break;
    case '':
      return "\\\"o";
      break;
    case '':
      return "\\\"u";
      break;
    case '':
      return "\\\"A";
      break;
    case '':
      return "\\\"O";
      break;
    case '':
      return "\\\"U";
      break;
    case '':
      return "\\'a";
      break;
    case '':
      return "\\'e";
      break;
    case '':
      return "\\'i";
      break;
    case '':
      return "\\'o";
      break;
    case '':
      return "\\'u";
      break;
    case '':
      return "\\`a";
      break;
    case '':
      return "\\`e";
      break;
    case '':
      return "\\`i";
      break;
    case '':
      return "\\`o";
      break;
    case '':
      return "\\`u";
      break;
    case '':
      return "\\'A";
      break;
    case '':
      return "\\'E";
      break;
    case '':
      return "\\'I";
      break;
    case '':
      return "\\'O";
      break;
    case '':
      return "\\'U";
      break;
    case '':
      return "\\`A";
      break;
    case '':
      return "\\`E";
      break;
    case '':
      return "\\`I";
      break;
    case '':
      return "\\`O";
      break;
    case '':
      return "\\`U";
      break;
    case '':
      return "\\ss ";
      break;
    default :
      m += c;
      return m;
    }
}
/** gibt LaTex-Text aus */
void LatexCode::printBody()
{
  *out   << "\\noindent\n"
         << "\\ttfamily\n";

  processStandardState();
  *out << "\\mbox{}\\\\"
       << "\n\\normalfont"<<endl;
}

void LatexCode::printFooter()
{
  *out << "\n\\end {document}\n"
       << "(* LaTeX generated by highlight "
       << HIGHLIGHT_VERSION
       << ", "
       << HIGHLIGHT_URL
       << " *)"
       << endl;
}

void LatexCode::printHeader(const string & title)
{
  *out << "\\documentclass{article}\n"
       << "\\usepackage{color}\n"
       << "\\usepackage{alltt}\n";
  *out << styleDefinition;
  *out << "\\begin{document}"
       << endl
       << "\\definecolor{bgcolor}{rgb}{"
       << docStyle->getBgColour() ->getLatexRedValue() << ","
       << docStyle->getBgColour()->getLatexGreenValue() << ","
       << docStyle->getBgColour()->getLatexBlueValue()
       << "}\n\\pagecolor{bgcolor}"
       << endl;
}



