/***************************************************************************
                          main.cpp  -  description
                             -------------------
    begin                : Die Apr 23 22:16:35 CEST 2002
    copyright            : (C) 2002 by Andr Simon
    email                : andre.simon1@gmx.de


   Highlight is a universal source code to HTML converter. Syntax  highlighting
   is formatted by Cascading Style Sheets. It's possible to easily enhance
   highlight's parsing database.

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <iostream>
#include <fstream>
#include <string>
#include <vector>

#include "cmdlineoptions.h"
#include "languagedefinition.h"
#include "htmlcode.h"
#include "rtfcode.h"
#include "latexcode.h"
#include "texcode.h"
#include "help.h"
#include "datadir.h"
#include "codeparser.h"
#include "directoryreader.h"
#include "version.h"


using namespace std;

//Gibt Versionsinformationen nach cout aus
void printVersionInfo()
{
  cout << "\n highlight version " << HIGHLIGHT_VERSION
       << "\n Copyright (C) 2002, 2003 Andre Simon <andre.simon1@gmx.de>"
       << "\n\n Artistic Style Classes (1.15.3)"
       << "\n Copyright (C) 1998-2002 Tal Davidson <davidsont@bigfoot.com>"
       << "\n\n DirStream Class"
#ifdef _WIN32
       << ", CommandLineParser Class"
#endif
       << "\n Copyright (C) 2002 Benjamin Kaufmann <hume@c-plusplus.de>"
       << "\n\n This software is released under the terms of the GNU General Public License."
       << "\n For more information about these matters, see the file named COPYING.\n"
       << endl;
}

void printBadInstallationInfo()
{
  cerr << "highlight: Data directory not found. Bad installation or wrong data-dir parameter."
#ifndef _WIN32
       << "\n\nCopy the highlight files (*.lang, *.style) in one of the following directories:\n"
       << "/usr/share/highlight/\n"
       << "/usr/local/share/highlight/\n"
       << "/opt/highlight/\n"
       << "/opt/bin/highlight/\n"
       << "$HOME/.highlight/\n\n"
#else
       << "\n\nCopy the highlight files (*.lang, *.style) in the highlight.exe root directory.\n"
#endif
       << "You may also set the data directory with --data-dir.\n";
}

bool listInstalledFiles(bool themes)
{
  // Liste der Eingabedateien
  vector <string> inFileList;
  unsigned int suffixLength=(themes)?6:5;
  string searchDir = (themes)?DataDir::getThemeDir()+"*.style":DataDir::getLangDefDir()+"*.lang";
  // Liste der Eingabedateien fuellen:
  bool directoryOK = DirectoryReader::getEntries (inFileList,searchDir);
  if (!directoryOK)
    {
      return false;
    }

  string  temp;

  cout << endl<< "  Installed "
       << ((themes)? "themes":"language definitions ")
       << "(located in "
       << ((themes)?DataDir::getThemeDir():DataDir::getLangDefDir())
       << ") :"<<endl << endl;


  for (unsigned int i=0;i< inFileList.size(); i++)
    {
      if (themes)
       temp = (inFileList[i]).substr(DataDir::getThemeDir().length());
      else
       temp = (inFileList[i]).substr(DataDir::getLangDefDir().length());
      cout << "  "<<temp.substr(0, temp.length()- suffixLength) << endl;
    }
  cout << endl <<"  Use name of the desired "
               << ((themes)?"theme":"language")
               << " with the -"
               << ((themes)?"s":"S")
               <<" option."<< endl << endl;
  return true;
}

void printParseError(enum highlight::ParseError err,const string & inFile, const string & outFile){
  switch (err){
 case highlight::BAD_OUTPUT:
  cerr << "highlight: Could not write to "
          << outFile
          << "."
          << endl;
   break;
case highlight::BAD_INPUT:
 cerr << "highlight: Input file "
          << inFile
          << " not found."
          << endl;
break;
default:;
  }
 }

/*********************************************/

int main(int argc, char *argv[])
{

  // Win32: Get application path
#ifdef _WIN32
  string m_AppPath = argv[0];
	string::size_type Pos = m_AppPath.find_last_of('\\');
	m_AppPath = m_AppPath.substr(0, Pos+1);
#endif

  //get command line options
  CmdLineOptions options(argc, argv);

  // set data directory path, where /langDefs anf /themes reside
#ifdef _WIN32
  DataDir::searchDataDir((options.newDataDir())?
                                options.getDataDir():m_AppPath );
#else
  // Highlight-Verzeichnis bestimmen
  if (! DataDir::searchDataDir((options.newDataDir())?
                                options.getDataDir():"" ))
    {
      printBadInstallationInfo();
      return EXIT_FAILURE;
    }
#endif

  // load entries from extensions.conf
#ifdef _WIN32
  options.loadFileExtensions(m_AppPath);
#else
   options.loadFileExtensions();
#endif

  if (options.printVersion())
    {
      printVersionInfo();
      return EXIT_SUCCESS;
    }

  if  (options.printHelp())
    {
      Help::printHelp(options.getErrMsg(), options.getHelpMsg());
      return EXIT_SUCCESS;
    }

  if (options.showThemes())
    {
      if (!listInstalledFiles(true))
        {
          cerr << "Could not access directory "
          <<  DataDir::getThemeDir() << ", aborted."<<endl;
          return EXIT_FAILURE;
        }
      return EXIT_SUCCESS;
    }
  if (options.showLangdefs())
    {
      if (!listInstalledFiles(false))
        {
          cerr << "Could not access directory "
          <<  DataDir::getLangDefDir() << ", aborted."<<endl;
          return EXIT_FAILURE;
        }
      return EXIT_SUCCESS;
    }



  if (options.getLanguage()==".lang")
    {
      cerr<<"highlight: Undefined language definition. Use -S option."<<endl;
      return EXIT_FAILURE;
    }

  // load language definition
  highlight::LanguageDefinition lang;
  bool langDefinitionFound = lang.load(options.getLanguage());

  if (!langDefinitionFound)
    {
      cerr << "highlight: Unknown source file extension \""
           << options.getLanguage().substr(0, options.getLanguage().length()-5)
           << "\"."<<endl;
      return EXIT_FAILURE;
    }

  //Debug-Informationen ausgeben (zum Testen von Sprachdefinitionsdateien)
  if (options.printDebugInfo())
    {
      cerr << "<HIGHLIGHT DEBUG>" << endl <<
      "LANGUAGE: " << options.getLanguage()  << endl <<
      "\nSYMBOLS: "  << lang.getSymbolString() << endl <<
      "\nKEYWORDS: " << flush;
      lang.printKeys();
      cerr << endl << "\nTYPES:    " << flush;
      lang.printTypes();
      cerr << endl << "</DEBUG>" << endl;
    }

  // determine number of spaces, to replace tabs
  int numberSpaces = (options.deleteTabs()) ? options.getNumberSpaces() : 0;

  highlight::CodeParser  *parser = NULL;

  // get an appropriate code parser instance
  if (options.texFormat())
    {
      parser = new TexCode(lang,
                           options.getStyleName(),
                           options.printLineNumbers(),
                           numberSpaces);
    }
  else if (options.latexFormat())
    {
      parser = new LatexCode(lang,
                             options.getStyleName(),
                             options.printLineNumbers(),
                             numberSpaces);
    }
  else if (options.rtfFormat())
    {
      parser = new RtfCode(lang,
                           options.getStyleName(),
                           options.printLineNumbers(),
                           numberSpaces);
    }
  else
    {
      parser = new HtmlCode  ( lang,
                               options.getStyleName(),
                               options.getCssOutFilename(),
                               options.includeCssDefinitions(),
                               options.printLineNumbers(),
                               options.attachLineAnchors(),
                               options.xhtmlFormat(),
                               numberSpaces);
       // print external css file
       bool cssOk= ((HtmlCode*)parser) -> printCSSFile(options.getCssInFilename(),
                                          options.getOutDirectory()+ options.getCssOutFilename());
       if (!cssOk){
         cerr << "highlight: Could not write "
              << options.getOutDirectory()
              << options.getCssOutFilename()<<"."<<endl;
       }
    }

  // enable Artistic Style code formatting and indentation
  if (options.formattingEnabled())
    {
      parser->setFormatter(options.getFormatStyle());
    }

  // list of input files
  vector <string> inFileList;

  string outfilePath;

  // fill input file list
  if (options.enableBatchMode())
    {
      bool directoryOK = DirectoryReader::getEntries (inFileList,
                                                      options.getBatchWildcard(),
                                                      options.batchRecursive());
      if (!directoryOK)
        {
          cerr << "highlight: No files matched the pattern "
               << options.getBatchWildcard() << "."<< endl;
          return EXIT_FAILURE;
        }
      else if (options.printIndexFile()){
         ((HtmlCode*)parser) -> printIndexFile(inFileList, options.getOutDirectory());
      }
    }
  else
    {
      // insert file name given by -i
      inFileList.push_back(options.getInFilename());

    }

  unsigned int count=1, size=inFileList.size();
  enum highlight::ParseError error;

  // iterate through file list and generate output
//  for (infilePath = inFileList.begin(); infilePath != inFileList.end(); infilePath++, count++)
    for (unsigned int i=0; i< inFileList.size(); i++, count++)
    {
      outfilePath = options.getOutDirectory();
      if (options.enableBatchMode()){
         string::size_type pos =
#ifdef _WIN32
                 static_cast<string>(inFileList[i]).find_last_of('\\');
#else
                 static_cast<string>(inFileList[i]).find_last_of('/');
#endif
         outfilePath += static_cast<string>(inFileList[i]).substr(pos+1);
         outfilePath += options.getOutFileSuffix();
         cout << " Writing file "<< count <<" of "<< size << ": "
              << outfilePath << endl;
      }
      else {
         outfilePath =  options.getOutFilename();
      }
      error = parser->printOutput(inFileList[i], outfilePath, options.fragmentOutput());
      if (error != highlight::PARSE_OK){
        printParseError(error, inFileList[i],outfilePath);
      }
    }
  delete parser;

  return EXIT_SUCCESS;
}


