/***************************************************************************
                          rtfcode.cpp  -  description
                             -------------------
    begin                : Die Jul 9 2002
    copyright            : (C) 2002 by Andr Simon
    email                : andre.simon1@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "rtfcode.h"
#include "version.h"
#include <iostream>
#include <fstream>
#include <string>
#include <sstream>

using namespace std;
using namespace highlight;

string  RtfCode::formatStyleAttributes( StyleColour * col)
{
  stringstream s;
  s  << "\\red"<<col->getRTFRedValue()
  << "\\green"<<col->getRTFGreenValue()
  << "\\blue"<<col->getRTFBlueValue()
  << ";";

  return s.str();
}

string  RtfCode::getOpenTag(int styleNumber,ElementStyle * elem)
{
  stringstream s;
  s  << "{\\cf"<<(styleNumber+1)<<"{";
  if (elem!=NULL){
    s << ( elem->isBold() ?"\\b " :"" )
      << ( elem->isItalic() ?"\\i " :"" )
      << ( elem->isUnderline() ?"\\ul " :"" );
  }
  return  s.str();
}

string  RtfCode::getCloseTag(ElementStyle * elem)
{
  stringstream s;
  if (elem!=NULL)  {
    s   << ( elem->isBold() ?"\\b0 " :"" )
        << ( elem->isItalic() ?"\\i0 " :"" )
        << ( elem->isUnderline() ?"\\ul0 " :"" );
  }
  s << "}}";
  return  s.str();
}



// Definition der RTF-Farbcodes, die zum Highlighting benutzt werden:

RtfCode::RtfCode( LanguageDefinition &langDef,const string &cssStyle,
                  bool printLineNumbers, int numSpaces)
    : CodeParser(langDef, printLineNumbers, cssStyle, numSpaces)
{
  styleTagOpen[0] =getOpenTag(STANDARD);
  styleTagOpen[1] =getOpenTag(KEYWORD, docStyle->getKeywordStyle());
  styleTagOpen[2] =getOpenTag(TYPE, docStyle->getTypeStyle());
  styleTagOpen[3] =getOpenTag(STRING, docStyle->getStringStyle());
  styleTagOpen[4] =getOpenTag(NUMBER, docStyle->getNumberStyle());
  styleTagOpen[5] =getOpenTag(SINGLE_LINE_COMMENT, docStyle->getCommentStyle());
  styleTagOpen[6] =getOpenTag(MULTI_LINE_COMMENT_BEGIN, docStyle->getCommentStyle());
  styleTagOpen[7] =getOpenTag(ESC_CHAR, docStyle->getEscapeCharStyle());
  styleTagOpen[8] =getOpenTag(DIRECTIVE_LINE, docStyle->getDirectiveStyle());
  styleTagOpen[9] =getOpenTag(DIRECTIVE_STRING, docStyle->getDirectiveStringStyle());
  styleTagOpen[10] =getOpenTag(LINENUMBER, docStyle->getLineStyle());

  styleTagClose[0] =getCloseTag();
  styleTagClose[1] =getCloseTag( docStyle->getKeywordStyle());
  styleTagClose[2] =getCloseTag( docStyle->getTypeStyle());
  styleTagClose[3] =getCloseTag(docStyle->getStringStyle());
  styleTagClose[4] =getCloseTag( docStyle->getNumberStyle());
  styleTagClose[5] =getCloseTag( docStyle->getCommentStyle());
  styleTagClose[6] =getCloseTag( docStyle->getCommentStyle());
  styleTagClose[7] =getCloseTag( docStyle->getEscapeCharStyle());
  styleTagClose[8] =getCloseTag( docStyle->getDirectiveStyle());
  styleTagClose[9] =getCloseTag(docStyle->getDirectiveStringStyle());
  styleTagClose[10] =getCloseTag( docStyle->getLineStyle());

  newLineTag = "\\par\\pard\n";
  spacer = " ";

  stringstream s;

  s << formatStyleAttributes(docStyle->getDefaultColour());
  s << formatStyleAttributes( docStyle->getKeywordColour());
  s << formatStyleAttributes(docStyle->getTypeColour());
  s << formatStyleAttributes(docStyle->getStringColour());
  s << formatStyleAttributes(docStyle->getNumberColour());
  s << formatStyleAttributes(docStyle->getCommentColour());
  s << formatStyleAttributes(docStyle->getCommentColour());
  s << formatStyleAttributes(docStyle->getEscapeCharColour());
  s << formatStyleAttributes(docStyle->getDirectiveColour());
  s << formatStyleAttributes(docStyle->getStringColour());
  s << formatStyleAttributes(docStyle->getDirectiveStringColour());
  s << formatStyleAttributes(docStyle->getLineColour());
  styleDefinition = s.str();
}

RtfCode::RtfCode()
{}
RtfCode::~RtfCode()
{}

void RtfCode::printHeader(const string & title)
{

  *out << "{\\rtf1\\ansi\\uc0 \\deff1"
  << "{\\fonttbl{\\f1\\fmodern\\fprq1\\fcharset0 Courier;}}"
  << "{\\colortbl;";
  *out <<  styleDefinition;
  *out << "}\n{\\info }\\paperw11900\\paperh16820\\margl560\\margr560\\margt840\\margb840\\widowctrl\\ftnbj\\aenddoc\\formshade \\fet0\\sectd"
  << "\\linex0\\endnhere\\plain\\f1\\fs20\n\\pard "
  << flush;
}

/** gibt RTF-Text aus */
void RtfCode::printBody()
{
  processStandardState();
}


void RtfCode::printFooter()
{
  *out << "\n}}"<<endl;
}

/** Gibt RTF-Code der Sonderzeichen zurueck */
string RtfCode::maskCharacter( char c)
{
  string m;
  switch (c)
    {
    case '}' :
      return "\\}";
      break;
    case '{' :
      return "\\{";
      break;
    case '\\' :
      return "\\\\";
      break;
    case '0':
      return "{0}";
      break;
    case '1':
      return "{1}";
      break;
    case '2':
      return "{2}";
      break;
    case '3':
      return "{3}";
      break;
    case '4':
      return "{4}";
      break;
    case '5':
      return "{5}";
      break;
    case '6':
      return "{6}";
      break;
    case '7':
      return "{7}";
      break;
    case '8':
      return "{8}";
      break;
    case '9':
      return "{9}";
      break;
    case '':
      return "\\'e4";
      break;
    case '':
      return "\\'f6";
      break;
    case '':
      return "\\'fc";
      break;
    case '':
      return "\\'c4";
      break;
    case '':
      return "\\'d6";
      break;
    case '':
      return "\\'dc";
      break;
    case '':
      return "\\'e1";
      break;
    case '':
      return "\\'e9";
      break;
    case '':
      return "\\'ed";
      break;
    case '':
      return "\\'f3";
      break;
    case '':
      return "\\'fa";
      break;
    case '':
      return "\\'e0";
      break;
    case '':
      return "\\'e8";
      break;
    case '':
      return "\\'ec";
      break;
    case '':
      return "\\'f2";
      break;
    case '':
      return "\\'f9";
      break;
    case '':
      return "\\'c1";
      break;
    case '':
      return "\\'c9";
      break;
    case '':
      return "\\'cd";
      break;
    case '':
      return "\\'d3";
      break;
    case '':
      return "\\'da";
      break;
    case '':
      return "\\'c0";
      break;
    case '':
      return "\\'c8";
      break;
    case '':
      return "\\'cc";
      break;
    case '':
      return "\\'d2";
      break;
    case '':
      return "\\'d9";
      break;
    case '':
      return "\\'df";
      break;
    default :
      m += c;
      return m;
    }
}
