/***************************************************************************
                          TexCode.cpp  -  description
                             -------------------
    begin                : Mit Jul 24 2002
    copyright            : (C) 2002 by Andr Simon
    email                : andre.simon1@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "texcode.h"
#include "stringtools.h"
#include "version.h"
#include <iostream>
#include <sstream>

using namespace highlight;

string  TexCode::getOpenTag(StyleColour *col, ElementStyle * elem)
{
  stringstream s;
  s << "{";
  if (elem!=NULL){
    if (elem->isBold())
      s << "\\bf";
    if (elem->isItalic())
      s << "\\it";
  }
  s  <<  "\\textColor{"
  << (col->getTexRedValue())<<" "
  << (col->getTexGreenValue())<<" "
  << (col->getTexBlueValue())<<" 0.0}";
  return  s.str();
}

TexCode::TexCode(highlight::LanguageDefinition &langDef,const string &cssStyle,
                 bool printLineNumbers, int numSpaces)
    : CodeParser(langDef, printLineNumbers, cssStyle, numSpaces)
{

  for (int i=0;i<NUMBER_OF_STYLES; i++)
    {
      styleTagClose[i] = "}";
    }
  styleTagOpen[STANDARD] =getOpenTag(docStyle->getDefaultColour());;  
  styleTagOpen[KEYWORD] =getOpenTag(docStyle->getKeywordColour(), docStyle->getKeywordStyle());
  styleTagOpen[TYPE] =getOpenTag(docStyle->getTypeColour(), docStyle->getTypeStyle());
  styleTagOpen[STRING] =getOpenTag(docStyle->getStringColour(), docStyle->getStringStyle());
  styleTagOpen[NUMBER] =getOpenTag(docStyle->getNumberColour(), docStyle->getNumberStyle());
  styleTagOpen[SINGLE_LINE_COMMENT] =getOpenTag(docStyle->getCommentColour(), docStyle->getCommentStyle());
  styleTagOpen[MULTI_LINE_COMMENT_BEGIN] =getOpenTag(docStyle->getCommentColour(), docStyle->getCommentStyle());
  styleTagOpen[ESC_CHAR] =getOpenTag(docStyle->getEscapeCharColour(), docStyle->getEscapeCharStyle());
  styleTagOpen[DIRECTIVE_LINE] =getOpenTag(docStyle->getDirectiveColour(), docStyle->getDirectiveStyle());
  styleTagOpen[DIRECTIVE_STRING] =getOpenTag(docStyle->getDirectiveStringColour(), docStyle->getDirectiveStringStyle());
  styleTagOpen[LINENUMBER] =getOpenTag(docStyle->getLineColour(), docStyle->getLineStyle());

  newLineTag = "\\par\n";
  spacer = "\\ ";
}

TexCode::TexCode()
{}
TexCode::~TexCode()
{}
/** Gibt Tex-Code der Sonderzeichen zurueck */
string TexCode:: maskCharacter( char c)
{
  string m;

  switch (c)
    {
    case '{':
    case '}':
      m += "$\\";
      m += c;
      m += "$";
      return m;
      break;
    case '^':
    case '_':
      m += "\\";
      m += c;
      m += "{}";
      return m;
      break;
    case '&':
    case '$':
    case '#':
    case '%':
      m+="\\";
      m += c;
      return m;
      break;
    case '\\':
      return "$\\backslash$";
      break;
    case '':
      return "\\\"a";
      break;
    case '':
      return "\\\"o";
      break;
    case '':
      return "\\\"u";
      break;

    case '':
      return "\\\"A";
      break;
    case '':
      return "\\\"O";
      break;
    case '':
      return "\\\"U";
      break;

    case '':
      return "\\'a";
      break;
    case '':
      return "\\'e";
      break;
    case '':
      return "\\'i";
      break;
    case '':
      return "\\'o";
      break;
    case '':
      return "\\'u";
      break;
    case '':
      return "\\`a";
      break;
    case '':
      return "\\`e";
      break;
    case '':
      return "\\`i";
      break;
    case '':
      return "\\`o";
      break;
    case '':
      return "\\`u";
      break;
    case '':
      return "\\'A";
      break;
    case '':
      return "\\'E";
      break;
    case '':
      return "\\'I";
      break;
    case '':
      return "\\'O";
      break;
    case '':
      return "\\'U";
      break;
    case '':
      return "\\`A";
      break;
    case '':
      return "\\`E";
      break;
    case '':
      return "\\`I";
      break;
    case '':
      return "\\`O";
      break;
    case '':
      return "\\`U";
      break;
    case '':
      return "\\ss ";
      break;
    case ' ':
      return "\\ ";
      break;

    default :
      string m;
      m += c;
      return m;
    }

}
/** gibt Tex-Text aus */
void TexCode::printBody()
{
  processStandardState();
}

void TexCode::printFooter()
{
  *out << "\n\\bye"
  << endl
  << "% TeX generated by Highlight "
  << HIGHLIGHT_VERSION
  << ", "
  << HIGHLIGHT_URL
  << endl << flush;
}

void TexCode::printHeader(const string & title)
{
  *out << "\\nopagenumbers"
       << "\\input colordvi \\tt\n"
       <<  "% The special option is not supported by all dvi drivers"
       << endl
       << "\\special{background rgb "
       << docStyle->getBgColour() ->getLatexRedValue() << " "
       << docStyle->getBgColour()->getLatexGreenValue() << " "
       << docStyle->getBgColour()->getLatexBlueValue() << "}"  
       << endl;
}
